/* aesthetic 'min edge intersect' - makes crossed edges uncross */
/* $Id: ae_min_edge_intersect.c,v 1.0 1992/10/23 06:38:56 coleman Exp $ */

#include <aesth.h>
#include <assert.h>

declare_aesth(min_edge_intersect);

define_setup(min_edge_intersect) {
    return;
}

static int ccw(aglo_real x0, aglo_real y0,
               aglo_real x1, aglo_real y1,
               aglo_real x2, aglo_real y2) {
    aglo_real dx1, dx2, dy1, dy2;
    dx1 = x1 - x0; dy1 = y1 - y0;
    dx2 = x2 - x0; dy2 = y2 - y0;
    if (dx1*dy2 > dy1*dx2) return +1;
    if (dx1*dy2 < dy1*dx2) return -1;
    if (dx1*dx2 < 0 || dy1*dy2 < 0) return -1;
    if (dx1*dx1+dy1*dy1 < dx2*dx2+dy2*dy2) return +1;
    return 0;
}

static int intersect(aglo_real x11, aglo_real y11,
                     aglo_real x12, aglo_real y12, /* line 1 */
                     aglo_real x21, aglo_real y21,
                     aglo_real x22, aglo_real y22) /* line 2 */
{
    return ccw(x11, y11, x12, y12, x21, y21) *
        ccw(x11, y11, x12, y12, x22, y22) <= 0 &&
        ccw(x21, y21, x22, y22, x11, y11) *
        ccw(x21, y21, x22, y22, x12, y12) <= 0;
}

static void ae_intersection(aglo_state_vector state,
                            aglo_state_vector gradient,
                            aglo_vertex p11, aglo_vertex p12,
                            aglo_vertex p21, aglo_vertex p22) {
    if (intersect(state[p11][0], state[p11][1],
                  state[p12][0], state[p12][1],
                  state[p21][0], state[p21][1],
                  state[p22][0], state[p22][1])) {
        aglo_point mid1, mid2;
        aglo_point delta;
        aglo_real mag;
        aglo_point force_delta;

        aglo_point_midpoint(mid1, state[p11], state[p12]);
        aglo_point_midpoint(mid2, state[p21], state[p22]);
        aglo_point_sub(delta, mid2, mid1);
        aglo_point_mag(&mag, delta);
        mag = fmax(mag, 1e-8);  /* avoid div by 0 */
        aglo_point_scalar_mult(force_delta, 1/mag, delta);

        aglo_point_dec(gradient[p11], force_delta);
        aglo_point_dec(gradient[p12], force_delta);
        aglo_point_inc(gradient[p21], force_delta);
        aglo_point_inc(gradient[p22], force_delta);
    }
}

define_aesth(min_edge_intersect) {
    unsigned i, j;
    aglo_edge_record p, q;

    /* this doesn't make any sense except in 2D */
    assert(AGLO_SPACE_ARITY == 2);

    /* all 4 endpoints are distinct */
    /* i p */
    /* i j q */
    for (i=0;i<graph->vertices;i++)
        for (p=graph->edge_table[i];p;p=p->next)
            if (i < p->tail)
                for (j=i+1;j<graph->vertices;j++)
                    if (j != p->tail)
                        for (q=graph->edge_table[j];q;q=q->next)
                            if (j < q->tail && p->tail != q->tail)
                                ae_intersection(state, gradient,
                                                i, p->tail,
                                                j, q->tail);
}
