/* aesthetic 'node edge repulsion' - makes node and edges spread apart */
/* $Id: ae_node_edge_repulsion.c,v 1.2 1992/10/23 06:37:57 coleman Exp $ */

#include <aesth.h>
#include <assert.h>
#include <defines.h>

declare_aesth(node_edge_repulsion);

define_setup(node_edge_repulsion) {
    return;
}

/* distance from point pn to edge determined by e1n and e2n */	
static void ae_point_linesegment_distance_2(aglo_graph graph, 
                                            aglo_state_vector state,
                                            aglo_state_vector gradient,
                                            aglo_vertex pn, aglo_vertex e1n, 
                                            aglo_vertex e2n) {
    aglo_point e1p, e2p, e1e2, e2e1;
    aglo_real dp_e2e1p, dp_e1e2p;

    aglo_point_sub(e1p, state[pn], state[e1n]);
    aglo_point_sub(e2p, state[pn], state[e2n]);
    aglo_point_sub(e1e2, state[e2n], state[e1n]);
    aglo_point_scalar_mult(e2e1, -1, e1e2);
    aglo_point_dot_product(&dp_e2e1p, e1p, e1e2);
    aglo_point_dot_product(&dp_e1e2p, e2p, e2e1);

    if (dp_e2e1p >= 0)          /* acute */
        if (dp_e1e2p >= 0)      /* acute */
        {
            aglo_point delta, e1part, e2part, e;
            aglo_real mag2;
            aglo_point force_delta;
            aglo_point half_force_delta;
	
            aglo_point_mag2(&mag2, e1e2);
            mag2 = fmax(mag2, 1e-8); /* avoid div by 0 */
            aglo_point_scalar_mult(e1part, dp_e1e2p/mag2, state[e1n]);
            aglo_point_scalar_mult(e2part, dp_e2e1p/mag2, state[e2n]);
            /* e is the point on e1e2 closest to p */
            aglo_point_add(e, e1part, e2part);
            aglo_point_sub(delta, state[pn], e); 

            aglo_point_mag2(&mag2, delta);
            mag2 = fmax(mag2, 1e-8); /* avoid div by 0 */

            aglo_point_scalar_mult(force_delta, 1/mag2, delta);
            aglo_point_scalar_mult(half_force_delta, 0.5, force_delta);

            aglo_point_inc(gradient[pn], force_delta);
            aglo_point_dec(gradient[e1n], half_force_delta);
            aglo_point_dec(gradient[e2n], half_force_delta);
        }
    /* really should handle the case when not both acute TBF */
}

define_aesth(node_edge_repulsion) {
    unsigned i, j;
    aglo_edge_record p;

    /* i < p */
    /* j != i; j != p */
    for (j=0; j<graph->vertices; j++)
        for (i=0; i<graph->vertices; i++)
            if (i != j)
                for (p=graph->edge_table[i]; p; p=p->next)
                    if (i < p->tail && j != p->tail)
                        ae_point_linesegment_distance_2(graph, state, gradient,
                                                        j, i, p->tail);
}
