/* attribute 'node level' - level of node (from root) */
/* $Id: at_node_level.c,v 1.1 1992/10/23 06:33:50 coleman Exp $ */

#include <at_node_level.h>
#include <assert.h>
#include <attribute.h>
#include <limits.h>

int at_node_level[AGLO_MAX_GRAPH_SIZE];
aglo_vertex at_level_nodes[AGLO_MAX_GRAPH_SIZE][AGLO_MAX_GRAPH_SIZE];
unsigned at_level_nodes_size[AGLO_MAX_GRAPH_SIZE];

aglo_boolean at_node_level_assigned[AGLO_MAX_GRAPH_SIZE]; /* gvlevel[i] valid? */

static int at_node_level_tree(aglo_graph graph, aglo_vertex v) {
    aglo_edge_record p;
    int kid_level=1;

    if (at_node_level_assigned[v]) return at_node_level[v];

    at_node_level_assigned[v] = true;
    at_node_level[v] = 0;

    for (p=graph->edge_table[v];p;p=p->next)
        if (p->forward) {
            int k = at_node_level_tree(graph, p->tail);
            if (k < kid_level)
                kid_level = k;
        }

    return at_node_level[v] = kid_level - 1;
}

static void at_contract_node_level_tree(aglo_graph graph) {
    int change;

    for (change=1;change;) {
        int i;
        aglo_edge_record p;

        change=0;
        for (i=0;i<graph->vertices;i++) {
            int max_p_level = -INT_MAX;
            for (p=graph->edge_table[i];p;p=p->next)
                if (!p->forward)
                    if (max_p_level < at_node_level[p->tail])
                        max_p_level = at_node_level[p->tail];
            if (max_p_level == -INT_MAX)
                continue;
            max_p_level++;
            if (max_p_level < at_node_level[i]) {
                at_node_level[i] = max_p_level;
                change=1;
            }
        }
    }
}

static void at_make_node_level_lists(aglo_graph graph, int level) {
    unsigned list=0;
    unsigned vcount=0;          /* doublecheck */
    unsigned i;

    for (i=0;i<graph->vertices;i++)
        at_node_level[i] -= level;

    level = 0;

    for (;;level++,list++)
    {
        unsigned tvcount = vcount;

        for (i=0;i<graph->vertices;i++)
            if (at_node_level[i] == level) {
                at_level_nodes[list][at_level_nodes_size[list]++] = i;
                vcount++;
            }
        if (vcount == graph->vertices)
            return;
        assert(vcount != tvcount);
    }
}

static void at_calculate_node_level(aglo_graph graph) {
    int i;
    int lev;
    int min_level = INT_MAX;

    /* initial assignement */
    for (i=0;i<graph->vertices;i++) {
        lev = at_node_level_tree(graph, i);
        if (lev < min_level)
            min_level = lev;
    }

    /* contraction */
    at_contract_node_level_tree(graph);

    at_make_node_level_lists(graph, min_level);
}

void at_setup_node_level(aglo_graph graph) {
    static unsigned long at_graph_sequence;

    if (UPDATE_GRAPH_ATTRIBUTE)
        at_calculate_node_level(graph);
}
