/* just spit out idealized Heywood graph (no aglo layout!!!) */
/* $Id: heywood.c,v 1.1 1993/05/26 23:21:47 coleman Exp $ */

#include <aglo.h>
#include <aglo_builtins.h>
#include <aglo_graph_io.h>
#include <math.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>

#ifdef sun
#include <floatingpoint.h>
#endif

time_t time(time_t *);

time_t start_time;
static aglo_boolean do_monitor;
static aglo_boolean do_end_sleep;

struct aglo_find_params default_params = { 100.0, 0.001, 1000, 2 };
aglo_find_params find_params = &default_params;

struct font_desc {
	char *flag;
	char *description;
};

struct font_desc font_desc[] = {
{ "nr", "node repulsion" },
{ "mel", "minimize edge length" },
{ "cp", "centripetal (repulsion from centroid)" },
{ "ner", "node/edge repulsion" },
{ "mei", "minimize edge intersections" },
{ "pl", "place parent to left of children" },
{ "mlv", "minimize intralevel variance" },
{ 0 }
};

struct aglo_font font[] = {
{ ae_node_repulsion, ae_setup_node_repulsion },
{ ae_min_edge_length, ae_setup_min_edge_length },
{ ae_centripetal, ae_setup_centripetal },
{ ae_node_edge_repulsion, ae_setup_node_edge_repulsion },
{ ae_min_edge_intersect, ae_setup_min_edge_intersect },
{ ae_parent_left, ae_setup_parent_left },
{ ae_min_level_variance, ae_setup_min_level_variance },
{ 0 }
};


	static void volatile
g_usage(void)
{
	unsigned i;
	
	fprintf(stderr, "usage: \n"
			"\t-it <iterations> (%u)\n"
			"\t-bt <beginning temperature> (%f)\n"
			"\t-et <ending temperature> (%f)\n"
			"\t-m [turn monitor on]\n"
			"\t-mr <monitor update rate (seconds)> (%u)\n"
			"\t-s [sleep until ^C at end]\n",
			find_params->k_iterations,
			find_params->k_begin_temp,
			find_params->k_end_temp,
			find_params->mon_delay
			);

	for (i=0; font[i].aesth_gradient; i++)
		fprintf(stderr, "\t-k%s <%s aesthetic>\n",
				font_desc[i].flag,
				font_desc[i].description);

	exit(1);
}

static double g_get_double_arg(unsigned i, int argc, char **argv) {
    double v;
    char *unused;
	
    if (i >= argc) g_usage();
    v = strtod(argv[i], &unused);
    if (*unused) g_usage();
    return v;
}

static void g_getopt(int argc, char **argv) {
    unsigned i;
    char *p;
    double d;

    for (i=1;i<argc;i++) {
        p = argv[i];
        switch (*p++) {
          case '-':
            switch(*p++) {
              case 'i':
                switch (*p++) {
                  case 's':
                    /* k_initial_size = g_get_double_arg(++i, argc, argv); */
                    break;
                  case 't':
                    if ((d = g_get_double_arg(++i, argc, argv)) < 1)
                        find_params->k_iterations = 0;
                    else
                        find_params->k_iterations = d;
                    break;
                  default:
                    g_usage();
                }
                break;
              case 'b':
                find_params->k_begin_temp = g_get_double_arg(++i, argc, argv);
                break;
              case 'e':
                find_params->k_end_temp = g_get_double_arg(++i, argc, argv);
                break;
              case 'c':
                switch (*p++)
                {
                  default:
                    g_usage();
                }
                break;
              case 'm':
                switch (*p++)
                {
                  case 0:
                    do_monitor = 1;
                    break;
                  case 'r':
                    find_params->mon_delay = g_get_double_arg(++i, argc, argv);
                    break;
                  default:
                    g_usage();
                }
                break;
              case 'n':
                /* k_normal_size = g_get_double_arg(++i, argc, argv); */
                break;
              case 's':
                do_end_sleep = 1;
                break;
              case 'k':
            {
                unsigned ai, found=0;

                for (ai=0;font[ai].aesth_gradient;ai++)
                {
                    if (!strcmp(p, font_desc[ai].flag))
                    {
                        font[ai].weight = g_get_double_arg(++i, argc, argv);
                        font[ai].enabled = true;
                        found = true;
                        break;
                    }
                }
                if (!found)
                    g_usage();
            }
                break;
              case 'l':
                (void) g_get_double_arg(++i, argc, argv);
                fprintf(stderr, "-l flag ignored\n");
                break;
              default:
                g_usage();
            }
            break;
          default:
            g_usage();
        }
    }
}

	static void
g_check_params(void)
{
	if (!find_params->k_iterations)
	{
		perror("iterations must be >0");
		exit (1);
	}
	if (find_params->k_end_temp <= 0
		|| find_params->k_end_temp >= find_params->k_begin_temp)
	{
		perror("require 0 < [end temp] < [begin temp]");
		exit(1);
	}
	if (find_params->mon_delay > 120)
		perror("large or negative monitor update rate specified");
}

	static void
initialize(int argc, char **argv)
{
	g_getopt(argc, argv);
	g_check_params();

#ifdef sun
	ieee_handler("set", "common", SIGFPE_ABORT);
#endif

	start_time = time(0);
}

	static void
print_graph_state(FILE *f, aglo_graph graph, aglo_state_vector state)
{
	fprintf(f, "BEGIN_STATE\n");
	aglo_output_edge_state(f, graph, state);
	fprintf(f, "END_STATE\n");
}

	static void
cleanup(void)
{
	printf("Elapsed time = %lu seconds\n", (unsigned long) (time(0) - start_time));
}

	static void
fudge_state(aglo_state_vector state)
{
	unsigned i;
	const int hn = 14;

	for (i=0;i<hn;i++)
	{
		double th = i * (2.0 / hn) + 0.1;

		state[i][0] = cos(M_PI*th);
		state[i][1] = sin(M_PI*th);
	}
}

	int
main(int argc, char **argv, char **envp)
{
	aglo_graph graph;
	aglo_state_vector state;
	
	initialize(argc, argv);
	aglo_startup(do_monitor);
	graph = aglo_read_graph(stdin);

	fudge_state(state);

	aglo_normalize_state(state, graph, state);
	print_graph_state(stdout, graph, state);
	aglo_cleanup(do_monitor);
	cleanup();
	exit(0);
}
