/* declarations for aglo library */
/* $Id: aglo.h,v 1.2 1992/10/29 09:03:09 coleman Exp $ */

#if !aglo_h
#define aglo_h 1

/* 2D or 3D (or whatever) layout */
#define AGLO_SPACE_ARITY (2)

typedef enum aglo_boolean { false=0, true=!false } aglo_boolean;
typedef double aglo_real;

/* max nodes in a graph */
#define AGLO_MAX_GRAPH_SIZE (500)

typedef aglo_real aglo_point[AGLO_SPACE_ARITY];
typedef aglo_point aglo_state_vector[AGLO_MAX_GRAPH_SIZE];
typedef unsigned aglo_vertex;


typedef struct aglo_edge_record
{
	aglo_vertex tail;			/* edge: head -> tail */
	aglo_boolean forward;		/* not tail -> head */
	struct aglo_edge_record *next;
} *aglo_edge_record;

typedef struct aglo_graph
{
	unsigned vertices;
	unsigned edges;
	aglo_edge_record edge_table[AGLO_MAX_GRAPH_SIZE];
} *aglo_graph;


typedef void aglo_aesth_gradient_fx(aglo_graph graph,
									aglo_state_vector state,
									aglo_state_vector gradient);
typedef aglo_aesth_gradient_fx *aglo_aesth_gradient;
typedef void aglo_aesth_setup_fx(aglo_graph);
typedef aglo_aesth_setup_fx *aglo_aesth_setup;

typedef struct aglo_font
{
	aglo_aesth_gradient aesth_gradient;
	aglo_aesth_setup aesth_setup;
	aglo_real weight;
	aglo_boolean enabled;
} *aglo_font;

typedef struct aglo_find_params
{
	aglo_real k_begin_temp;
	aglo_real k_end_temp;
	unsigned k_iterations;
	unsigned mon_delay;
} *aglo_find_params;


unsigned long aglo_graph_sequence; /* bumped when graph changed */
unsigned long aglo_state_sequence; /* bumped when state changed */

extern void aglo_startup(aglo_boolean do_monitor);
extern void aglo_find_layout(aglo_graph graph, aglo_font font,
							 aglo_state_vector state, aglo_find_params p,
							 aglo_boolean random_start,
							 aglo_boolean do_monitor); 
extern void aglo_normalize_state(aglo_state_vector norm_state,
								 aglo_graph graph, aglo_state_vector state);
extern void aglo_cleanup(aglo_boolean do_monitor);

#endif
