/* read graphs and write states, etc. */
/* $Id: graph_io.c,v 1.1 1993/05/26 23:19:38 coleman Exp $ */

#include <aglo.h>
#include <defines.h>
#include <stdio.h>
#include <system.h>


static aglo_boolean g_register_vertex(aglo_graph graph, aglo_vertex v) {
    if (!v) {
        perror("0 is invalid vertex");
        return false;
    }
    graph->vertices = max(graph->vertices, v);
    if (graph->vertices > AGLO_MAX_GRAPH_SIZE) {
        perror("vertex allocation exceeded");
        return false;
    }
    return true;
}

static aglo_boolean g_add_edge(aglo_graph graph, aglo_vertex v1, 
                               aglo_vertex v2, aglo_boolean forward) {
    aglo_edge_record gp = (aglo_edge_record) zmalloc(sizeof(*gp));
    aglo_edge_record front = graph->edge_table[--v1];

    gp->tail = --v2;
    gp->next = front;
    gp->forward = forward;
    graph->edge_table[v1] = gp;
    return true;
}

#if 0
/* read an attribute string */
static void g_read_attributes(double *attr) {
    int s, c;
    unsigned i;

    while (c = getc(stdin)) {
        if (c == EOF) return;
        if (isspace(c)) continue;
        if (c == '(') break;
        ungetc(c, stdin);
        return;
    }

    for (i=0;;i++,attr++) {
        s = scanf(" %lf", attr);
        if (s == EOF) {
            perror("invalid attribute format 1");
            exit(1);
        }
        if (s != 1) break;
        if (i >= GATTRN-1) {
            perror("attribute list too big");
            exit(1);
        }
    }

    while (c = getc(stdin)) {
        if (c == EOF) return;
        if (isspace(c)) continue;
        if (c == ')') break;
        perror("invalid attribute format 2");
        exit(1);
    }
}
#endif

aglo_graph aglo_read_graph(FILE *f) {
	aglo_vertex v1, v2;
	int s;
	aglo_graph graph = (aglo_graph) zmalloc(sizeof(*graph));

	while ((s = fscanf(f, " %u %u", &v1, &v2)) != EOF) {
            switch (s) {
              case 1:           /* vertex */
                if (!g_register_vertex(graph, v1))
                    return false;
                /* g_read_attributes(&gvattr[v1][0]); */
                break;
              case 2:           /* edge */
                if (v1 == v2) perror("self edge ignored");
                else {
                    /* double attr[GATTRN] = { 0.0 }; */
                    if (!g_register_vertex(graph, v1)
                        || !g_register_vertex(graph, v2)
                        || !g_add_edge(graph, v1, v2, true)
                        || !g_add_edge(graph, v2, v1, false)) return false;
                    graph->edges++;
                }
                break;
              default:
                perror("invalid input format");
                return 0;
            }
	}
	return graph;
}

void aglo_output_edge_state(FILE *f, aglo_graph graph, aglo_state_vector state)
{
    unsigned i, j;
    aglo_edge_record p;

    /* print each edge only once (no self-edges) */
    for (i=0;i<graph->vertices;i++)
        for (p=graph->edge_table[i];p;p=p->next)
            if (i < p->tail) {
                for (j=0;j<AGLO_SPACE_ARITY;j++)
                    fprintf(f, "%f ", state[i][j]);
                fprintf(f, "\n");
                for (j=0;j<AGLO_SPACE_ARITY;j++)
                    fprintf(f, "%f ", state[p->tail][j]);
                fprintf(f, "\n\n");
            }
}

