/* point manipulation routines */
/* $Id: point.c,v 1.1 1993/05/26 23:19:38 coleman Exp $ */

#include <point.h>

#include <aglo.h>
#include <defines.h>
#include <math.h>

	void
aglo_point_add(aglo_point result, aglo_point arg1, aglo_point arg2) {
    unsigned i;

    for(i=0;i<AGLO_SPACE_ARITY;i++)
        result[i] = arg1[i] + arg2[i];
}

void aglo_point_sub(aglo_point result, aglo_point arg1, aglo_point arg2) {
    unsigned i;

    for(i=0; i<AGLO_SPACE_ARITY; i++) result[i] = arg1[i] - arg2[i];
}

void aglo_point_inc(aglo_point result, aglo_point arg) {
    unsigned i;

    for(i=0;i<AGLO_SPACE_ARITY;i++)
        result[i] += arg[i];
}

void aglo_point_dec(aglo_point result, aglo_point arg) {
    unsigned i;

    for(i=0;i<AGLO_SPACE_ARITY;i++)
        result[i] -= arg[i];
}

void aglo_point_mag2(aglo_real *result, aglo_point arg) {
    unsigned i;

    *result = 0;
    for(i=0;i<AGLO_SPACE_ARITY;i++)
        *result += sqr(arg[i]);
}

void aglo_point_mag(aglo_real *result, aglo_point arg) {
    aglo_point_mag2(result, arg);
    *result = sqrt(*result);
}

void aglo_point_midpoint(aglo_point result, aglo_point arg1, aglo_point arg2) {
    unsigned i;

    for(i=0;i<AGLO_SPACE_ARITY;i++)
        result[i] = (arg1[i] + arg2[i]) / 2;
}

void aglo_point_iso_frame(aglo_point min_iso_frame, aglo_point max_iso_frame,
                          aglo_point min_frame, aglo_point max_frame) {
    aglo_point frame_sides, frame_center, iso_radius;
    aglo_real max_side;
    unsigned i;

    aglo_point_sub(frame_sides, max_frame, min_frame);
    aglo_point_midpoint(frame_center, min_frame, max_frame);
    max_side = frame_sides[0];
    for(i=1;i<AGLO_SPACE_ARITY;i++)
        max_side = fmax(max_side, frame_sides[i]);
    for(i=0;i<AGLO_SPACE_ARITY;i++)
        iso_radius[i] = max_side/2;
    aglo_point_sub(min_iso_frame, frame_center, iso_radius);
    aglo_point_add(max_iso_frame, frame_center, iso_radius);
}

void aglo_point_scalar_mult(aglo_point result, aglo_real scalar_arg,
                            aglo_point point_arg) {
    unsigned i;

    for(i=0;i<AGLO_SPACE_ARITY;i++)
        result[i] = scalar_arg * point_arg[i];
}

void aglo_point_dot_product(aglo_real *result, aglo_point arg1,
                            aglo_point arg2) {
    unsigned i;

    *result = 0;
    for(i=0;i<AGLO_SPACE_ARITY;i++)
        *result += arg1[i] * arg2[i];
}

void aglo_point_assign(aglo_point result, aglo_point arg) {
    unsigned i;

    for(i=0;i<AGLO_SPACE_ARITY;i++)
        result[i] = arg[i];
}

void aglo_point_zero(aglo_point result) {
    unsigned i;

    for(i=0;i<AGLO_SPACE_ARITY;i++)
        result[i] = 0;
}
